;;; ACP3 process description
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: March 1988
;;; Updated: Sun Aug 23 18:24:24 1998 by Jeff Dalton
;;; Copyright: (c) 1998, AIAI, University of Edinburgh

;;; Orderings:

;;; Each list in the orderings represents a sequence.  Consider adjacent
;;; elements A and B in such a list.  Each node in A is linked before
;;; each node in B.  For example, (1 (2 3) 4) is equivalent to the four
;;; separate lists (1 2), (1 3), (2 4), (3 4).  The more compact notation
;;; is much easier to understand than the separate lists, once you get
;;; used to it.

;;; Remember that you can't tell what orderings apply to a node by looking
;;; at the schema for that node's action.  You have to look at the schema
;;; for the *parent* action.

;;; Terminology in properties:

;;; An or-join becomes ready for execution as soon as any one of its
;;; immediate predecessors finishes execution.  By default, nodes are
;;; and-joins and require that all of their predecessors finish.

;;; When a decision-split finishes execution, all of its immediate
;;; successors become ready, but only one of them can execute.
;;; Only decision-splits can have more than one immediate successor.

;;; All links back to earlier nodes must be to loop-entries.  The node
;;; numbering in schemas must be such as to allow this to be detected
;;; by treating "earlier" as "lower numbered node in the schema",
;;; without any clever graph processing.

;;; There should be only one node that loops back to a given loop-entry.
;;; That's needed to make it easier to determine which nodes are in the
;;; loop.

;;; Some "overall" cells have a status that does not depend on their
;;; own row.  Instead, they look at an earlier row to determine how
;;; many COAs have completed that earlier action.  The "overall" cell
;;; becomes "ready" when enough have done so.  In such cases, the
;;; earlier action is specified by ("overall-collects" action-name)
;;; and the required minimum number by ("overall-at-least" number).
;;; ("overall-at-least" :all) means that the action must have
;;; completed for all currently existing COAs.

;;; Some other things that are true:

;;; There are no ordering constraints apart from the ones specified in
;;; the schemas.  Therefore all before-after order-links will be between
;;; sibling nodes.  There are no links between nodes at different levels,
;;; and, if we consider the subtrees under two sibling nodes, there are
;;; no links between a node in one subtree and a node in the other.

;;; A given schema can be reached by expansion in only one way.  We could
;;; therefore always identify a unique parent schema, if we wanted to.

(define-process ("acp3")
  (nodes
    (1 ("develop JFACC guidance" ?coa))
    (2 ("develop targets" ?coa))
    (3 ("formalize planning advice" ?coa))
    (4 ("develop COA" ?coa))
    (5 ("decide to explore alternative COA(s)" ?coa))
    (6 ("decide to brief on COA(s)" ?coa))
    (7 ("brief on COA(s)")))
  (orderings
    (1 (2 3) 4 (5 6))
    (6 7)
    (5 1)))

;;; 0: develop JFACC guidance

(define-action ("develop JFACC guidance" ?coa)
  (nodes
    (1 ("formalize JFACC objectives" ?coa))
    (2 ("identify COGs" ?coa))
    (3 ("decompose objectives to tasks" ?coa))
    (4 ("assign MOMs" ?coa)))
  (orderings
    (1 2 3 4)
    (1 4))
  (properties
    ("loop-entry" "true"))
  )

(define-action ("formalize JFACC objectives" ?coa)
  )

(define-action ("identify COGs" ?coa)
  )

(define-action ("decompose objectives to tasks" ?coa)
  )

(define-action ("assign MOMs" ?coa)
  (properties
    ("or-join" "true")))

;;; 0: develop targets

(define-action ("develop targets" ?coa)
  )

;;; 0: formalize planning advice

(define-action ("formalize planning advice" ?coa)
  )

;;; 0: develop COA

(define-action ("develop COA" ?coa)
  (nodes
    (1 ("create COA" ?coa))
    (2 ("assign weapons systems" ?coa))
    (3 ("create support plan" ?coa))
    (4 ("review COA" ?coa))
    (5 ("monitor COA-creation process" ?coa)))
  (orderings
    (1 2 3 4))
  (properties
    ;("or-join" "true")			;really? /\/
    ("decision-split" "true"))
  )

;;; 1: create COA

(define-action ("create COA" ?coa)
  (nodes
    (1 ("execute plan-cycle" ?coa))
    (2 ("approve final COA" ?coa)))
  (orderings
    (1 2)
    (1 1))
  )

;;; 2: exectute plan-cycle

(define-action ("execute plan-cycle" ?coa)
  (nodes
    (1 ("create COA to one more level" ?coa))
    (2 ("approve COA" ?coa))
    (3 ("reject COA" ?coa)))
  ; /\/: There should not be a loop back from 3 to 1.  Instead, there
  ; should be a loop back to a new node that "does something different"
  ; at the current level.
  ; (orderings
  ;   (1 2)
  ;   (1 3 1))
  (orderings (1 (2 3)))
  (properties
    ("decision-split" "true")
    ("loop-entry" "true"))
  )

(define-action ("create COA to one more level" ?coa)
  (properties
    ("decision-split" "true")
    ; /\/: There should not be a loop back to this node.  (See above.)
    ; ("loop-entry" "true")
    )
  )

(define-action ("approve COA" ?coa)
  )

(define-action ("reject COA" ?coa)
  (properties
    ("treat as loopback" "true")) ;no link shows it loops back (see above) /\/
  )

;;; 2: approve final COA

(define-action ("approve final COA" ?coa)
  )

;;; 1: assign weapons systems

(define-action ("assign weapons systems" ?coa)
  )

;;; 1: create support plan

(define-action ("create support plan" ?coa)
  (nodes
    (1 ("execute support-plan-cycle" ?coa))
    (2 ("approve final support plan" ?coa)))
  (orderings
    (1 2)
    (1 1))
  )

;;; 2: execute support-plan-cycle

(define-action ("execute support-plan-cycle" ?coa)
  (nodes
    (1 ("create support plan to one more level" ?coa))
    (2 ("compose schedule" ?coa))
    (3 ("approve support plan" ?coa))
    (4 ("reject support plan" ?coa)))
  ; /\/:  There should not be a loop back from 4 to 1.  Instead, there
  ; should be a loop back to a new node that "does something different"
  ; at the current level.
  ; (orderings
  ;   (1 2 3)
  ;   (2 4 1))
  (orderings (1 2 (3 4)))
  (properties
    ("decision-split" "true")
    ("loop-entry" "true"))
  )

(define-action ("create support plan to one more level" ?coa)
  ; /\/: There should not be a loop back to this node.  (See above.)
  ; (properties
  ;   ("loop-entry" "true"))
  )

(define-action ("compose schedule" ?coa)
  (properties
    ("decision-split" "true"))
  )

(define-action ("approve support plan" ?coa)
  )

(define-action ("reject support plan" ?coa)
  (properties
    ("treat as loopback" "true")) ;no link shows it loops back (see above) /\/
  )

;;; 2: approve final support plan

(define-action ("approve final support plan" ?coa)
  )

;;; 1: review COA

(define-action ("review COA" ?coa)
  (nodes
    (1 ("critique COA by simulation" ?coa))
    (2 ("critique COA by resource-comparison" ?coa))
    (3 ("critique COA by bed-down evaluation" ?coa))
    (4 ("critique COA by quality measures" ?coa)))
  (orderings
    ;; all in parallel
    )
  )

(define-action ("critique COA by simulation" ?coa)
  )

(define-action ("critique COA by resource-comparison" ?coa)
  (properties
    ("initial-status" "N/A")
    ("overall-collects" "create support plan")
    ("overall-at-least" 2))
  )

(define-action ("critique COA by bed-down evaluation" ?coa)
  )

(define-action ("critique COA by quality measures" ?coa)
  )

;;; 1: monitor COA-creation process

(define-action ("monitor COA-creation process" ?coa)
  )

;;; 0: decide to explore alternative COA(s)

(define-action ("decide to explore alternative COA(s)" ?coa)
  ;; /\/: Made 1-per-coa to fit better with how ViewMatrix behaves.
  (properties
    ("initial-status" "N/A")		;really?  /\/
    ("overall-collects" "develop COA")
    ("overall-at-least" :all))
  )

;;; 0: decide to brief on COA(s)

(define-action ("decide to brief on COA(s)" ?coa)
  ;; /\/: Made 1-per-coa to fit better with how ViewMatrix behaves.
  (properties
    ("overall-collects" "develop COA")
    ("overall-at-least" 1))
  )

;;; 0: brief on COA(s)

(define-action ("brief on COA(s)")
  (nodes
    (1 ("create briefing"))
    (2 ("communicate briefing")))
  (orderings
    (1 2)))

(define-action ("create briefing")
  )

(define-action ("communicate briefing")
  )

;;; End
